@@ -1,4 +1,4 @@
-# Copyright 2001-2006, Paul Johnson (pjcj@cpan.org)
+# Copyright 2001-2007, Paul Johnson (pjcj@cpan.org)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -10,13 +10,13 @@ package Devel::Cover;
 use strict;
 use warnings;
 
-our $VERSION = "0.58";
+our $VERSION = "0.60";
 
 use DynaLoader ();
 our @ISA = "DynaLoader";
 
-use Devel::Cover::DB  0.58;
-use Devel::Cover::Inc 0.58;
+use Devel::Cover::DB  0.60;
+use Devel::Cover::Inc 0.60;
 
 use B qw( class ppname main_cv main_start main_root walksymtable OPf_KIDS );
 use B::Debug;
@@ -63,6 +63,7 @@ my $Pod = $INC{"Pod/Coverage/CountParents.pm"} ? "Pod::Coverage::CountParents" :
 my %Pod;                                 # Pod coverage data.
 
 my @Cvs;                                 # All the Cvs we want to cover.
+my @Subs;                                # All the subs we want to cover.
 my $Cv;                                  # Cv we are looking in.
 my $Sub_name;                            # Name of the sub we are looking in.
 
@@ -89,6 +90,7 @@ BEGIN
     ($File, $Line, $Collect) = ("", 0, 1);
     $Silent = ($ENV{HARNESS_PERL_SWITCHES} || "") =~ /Devel::Cover/ ||
               ($ENV{PERL5OPT}              || "") =~ /Devel::Cover/;
+    *OUT = $ENV{DEVEL_COVER_DEBUG} ? *STDERR : *STDOUT;
 }
 
 if (0 && $Config{useithreads})
@@ -99,15 +101,15 @@ if (0 && $Config{useithreads})
 
     my $original_join;
     BEGIN { $original_join = \&threads::join }
-    # print "original_join: $original_join\n";
+    # print STDERR "original_join: $original_join\n";
 
-    # $original_join = sub { print "j\n" };
+    # $original_join = sub { print STDERR "j\n" };
 
     # sub threads::join
     *threads::join = sub
     {
-        # print "threads::join- ", \&threads::join, "\n";
-        # print "original_join- $original_join\n";
+        # print STDERR "threads::join- ", \&threads::join, "\n";
+        # print STDERR "original_join- $original_join\n";
         my $self = shift;
         print STDERR "(joining thread ", $self->tid, ")\n";
         my @ret = $original_join->($self, @_);
@@ -125,7 +127,7 @@ if (0 && $Config{useithreads})
         $original_destroy->($self, @_);
     };
 
-    # print "threads::join: ", \&threads::join, "\n";
+    # print STDERR "threads::join: ", \&threads::join, "\n";
 
     my $new = \&threads::new;
     *threads::new = *threads::create = sub
@@ -137,12 +139,12 @@ if (0 && $Config{useithreads})
         $new->($class,
                sub
                {
-                   print "Starting thread\n";
+                   print STDERR "Starting thread\n";
                    set_coverage(keys %Coverage);
                    my $ret = [ $sub->(@_) ];
-                   print "Ending thread\n";
+                   print STDERR "Ending thread\n";
                    report() if $Initialised;
-                   print "Ended thread\n";
+                   print STDERR "Ended thread\n";
                    $wantarray ? @{$ret} : $ret->[0];
                },
                @_
@@ -171,7 +173,7 @@ BEGIN { $^P = 0x004 | 0x100 | 0x200 }
         {
             delete $Coverage{pod};  # Pod::Coverage unavailable
             $nopod = <<EOM;
-    Pod coverage is unvailable.  Please install Pod::Coverage from CPAN.
+    Pod coverage is unavailable.  Please install Pod::Coverage from CPAN.
 EOM
         }
 
@@ -179,7 +181,7 @@ EOM
         @coverage = get_coverage();
         my $last = pop @coverage || "";
 
-        print STDOUT __PACKAGE__, " $VERSION: Collecting coverage data for ",
+        print OUT __PACKAGE__, " $VERSION: Collecting coverage data for ",
               join(", ", @coverage),
               @coverage ? " and " : "",
               "$last.\n",
@@ -205,14 +207,14 @@ EOM
     my $run_end = 0;
     sub first_end
     {
-        # print "**** END 1 - $run_end\n";
+        # print STDERR "**** END 1 - $run_end\n";
         set_last_end() unless $run_end++
     }
 
     my $run_init = 0;
     sub first_init
     {
-        # print "**** INIT 1 - $run_init\n";
+        # print STDERR "**** INIT 1 - $run_init\n";
         collect_inits() unless $run_init++
     }
 }
@@ -226,8 +228,8 @@ sub last_end
 
 {
     no warnings "void";  # avoid "Too late to run ... block" warning
-    INIT  {}  # dummy sub to make sure PL_endav  is set up and populated
-    END   {}  # dummy sub to make sure PL_initav is set up and populated
+    INIT  {}  # dummy sub to make sure PL_initav is set up and populated
+    END   {}  # dummy sub to make sure PL_endav  is set up and populated
     CHECK { set_first_init_and_end() }  # we really want to be first
 }
 
@@ -252,7 +254,7 @@ sub import
     my $class = shift;
 
     my @o = (@_, split ",", $ENV{DEVEL_COVER_OPTIONS} || "");
-    # print __PACKAGE__, ": Parsing options from [@_]\n";
+    # print STDERR __PACKAGE__, ": Parsing options from [@_]\n";
 
     my $blib = -d "blib";
     @Inc     = () if "@o" =~ /-inc /;
@@ -404,8 +406,9 @@ sub normalised_file
 
     my $f = $file;
     $file =~ s/ \(autosplit into .*\)$//;
-    # print "file is <$file>\n";
-    # use Data::Dumper; print "file is <$file>\ncoverage is ", Dumper coverage(0);
+    # print STDERR "file is <$file>\n";
+    # use Data::Dumper;
+    # print STDERR "file is <$file>\ncoverage: ", Dumper coverage(0);
     if (exists coverage(0)->{module} && exists coverage(0)->{module}{$file} &&
         !File::Spec->file_name_is_absolute($file))
     {
@@ -483,10 +486,11 @@ sub use_file
     for (@Ignore_re) { return $Files{$file} = 0 if $f =~ $_ }
     for (@Inc_re)    { return $Files{$file} = 0 if $f =~ $_ }
 
-    # system "pwd; ls -l $file";
+    # system "pwd; ls -l '$file'";
     $Files{$file} = -e $file ? 1 : 0;
-    warn __PACKAGE__ . qq(: Can't find file "$file": ignored.\n)
-        unless $Files{$file} || $Silent || $file =~ /\(eval \d+\)/;
+    warn __PACKAGE__ . qq(: Can't find file "$file" (@_): ignored.\n)
+        unless $Files{$file} || $Silent || $file =~ /\(eval \d+\)/ ||
+               $file eq "../../lib/Storable.pm";
 
     $Files{$file}
 }
@@ -502,7 +506,7 @@ sub check_file
 
     my $file = $op->file;
     my $use  = use_file($file);
-    # printf "%6s $file\n", $use ? "use" : "ignore";
+    # printf STDERR "%6s $file\n", $use ? "use" : "ignore";
 
     $use
 }
@@ -512,7 +516,7 @@ sub B::GV::find_cv
     my $cv = $_[0]->CV;
     return unless $$cv;
 
-    # print "find_cv $$cv\n" if check_file($cv);
+    # print STDERR "find_cv $$cv\n" if check_file($cv);
     push @Cvs, $cv if check_file($cv);
     push @Cvs, grep check_file($_), $cv->PADLIST->ARRAY->ARRAY
         if $cv->can("PADLIST") &&
@@ -529,7 +533,7 @@ sub sub_info
     {
         return unless $cv->GV->can("SAFENAME");
         $name = $cv->GV->SAFENAME;
-        # print "--[$name]--\n";
+        # print STDERR "--[$name]--\n";
         $name =~ s/(__ANON__)\[.+:\d+\]/$1/ if defined $name;
     }
     my $root = $cv->ROOT;
@@ -552,7 +556,7 @@ sub sub_info
 
 sub check_files
 {
-    # print "Checking files\n";
+    # print STDERR "Checking files\n";
 
     @Cvs = grep check_file($_), B::main_cv->PADLIST->ARRAY->ARRAY;
 
@@ -571,6 +575,7 @@ sub check_files
             local ($Line, $File);
             get_location($start);
             $line = $Line;
+            # print STDERR "$name - $File:$Line\n";
         }
         ($line, $name)
     };
@@ -580,6 +585,13 @@ sub check_files
            map  [ $_, $l->($_) ],
            grep !$seen_cv{$$_}++,
            @Cvs;
+
+    # Hack to bump up the refcount of the subs.  If we don't do this then the
+    # subs in some modules don't seem to be around when we get to looking at
+    # them.  I'm not sure why this is, and it seems to me that this hack could
+    # affect the order of destruction, but I've not seen any problems.  Yet.
+    # object_2svref doesn't exist before 5.8.1.
+    @Subs = map $_->object_2svref, @Cvs if $] >= 5.008001;
 }
 
 sub report
@@ -600,7 +612,8 @@ sub report
     $Structure      = Devel::Cover::DB::Structure->new(base => $DB);
     $Structure->read_all;
     $Structure->add_criteria(@collected);
-    # use Data::Dumper; print STDERR "Start structure", Dumper $Structure;
+    # use Data::Dumper; $Data::Dumper::Indent = 1;
+    # use Data::Dumper; print STDERR "Start structure: ", Dumper $Structure;
 
     # print STDERR "Processing cover data\n@Inc\n";
     $Coverage = coverage(1) || die "No coverage data available.\n";
@@ -622,17 +635,17 @@ sub report
         get_cover($_)
             for get_ends()->isa("B::AV") ? get_ends()->ARRAY : ();
     }
-    # print "--- @Cvs\n";
+    # print STDERR "--- @Cvs\n";
     get_cover($_) for @Cvs;
 
     my %files;
     $files{$_}++ for keys %{$Run{count}}, keys %{$Run{vec}};
     for my $file (sort keys %files)
     {
-        # print "looking at $file\n";
+        # print STDERR "looking at $file\n";
         unless (use_file($file))
         {
-            # print "deleting $file\n";
+            # print STDERR "deleting $file\n";
             delete $Run{count}->{$file};
             delete $Run{vec}  ->{$file};
             $Structure->delete_file($file);
@@ -649,7 +662,7 @@ sub report
         $Structure->store_counts($file);
     }
 
-    # use Data::Dumper; print STDERR "End structure", Dumper $Structure;
+    # use Data::Dumper; print STDERR "End structure: ", Dumper $Structure;
 
     my $run = time . ".$$." . sprintf "%05d", rand 2 ** 16;
     my $cover = Devel::Cover::DB->new
@@ -669,7 +682,7 @@ sub report
 
     $cover->{db} = $DB;
 
-    print STDOUT __PACKAGE__, ": Writing coverage database to $DB\n"
+    print OUT __PACKAGE__, ": Writing coverage database to $DB\n"
         unless $Silent;
     $cover->write;
     $cover->print_summary if $Summary && !$Silent;
@@ -682,7 +695,7 @@ sub add_subroutine_cover
     get_location($op);
     return unless $File;
 
-    # print "Subroutine $Sub_name $Line:$File: ", $op->name, "\n";
+    # print STDERR "Subroutine $Sub_name $Line:$File: ", $op->name, "\n";
 
     my $key = get_key($op);
     my $val = $Coverage->{statement}{$key} || 0;
@@ -804,7 +817,8 @@ sub add_condition_cover
         $name = $r->first->name if $name eq "sassign";
         # TODO - exec?  any others?
         # print STDERR "Name [$name]\n";
-        if ($c->[5] || $name =~ /^const|s?refgen|gelem|die|undef|bless$/)
+        if ($c->[5] || $name =~
+            /^const|s?refgen|gelem|die|undef|bless|anon(?:list|hash)$/)
         {
             $c = [ $c->[3], $c->[1] + $c->[2] ];
             $count = 2;
@@ -878,7 +892,7 @@ sub deparse
 
         my $name = $op->can("name") ? $op->name : "Unknown";
 
-        # print "$class:$name at $File:$Line\n";
+        # print STDERR "$class:$name at $File:$Line\n";
 
         {
             # Collect everything under here.
@@ -886,7 +900,7 @@ sub deparse
             $deparse = eval { $Original{deparse}->($self, @_) };
             $deparse =~ s/^\010+//mg if defined $deparse;
             $deparse = "Deparse error: $@" if $@;
-            # print "Collect Deparse $op $$op => <$deparse>\n";
+            # print STDERR "Collect Deparse $op $$op => <$deparse>\n";
         }
 
         # Get the coverage on this op.
@@ -947,7 +961,7 @@ sub deparse
         $deparse = eval { $Original{deparse}->($self, @_) };
         $deparse =~ s/^\010+//mg if defined $deparse;
         $deparse = "Deparse error: $@" if $@;
-        # print "Deparse => <$deparse>\n";
+        # print STDERR "Deparse => <$deparse>\n";
     }
 
     $deparse
@@ -1032,11 +1046,11 @@ sub get_cover
 
     ($Sub_name, my $start) = sub_info($cv);
 
-    # print "get_cover: <$Sub_name>\n";
+    # print STDERR "get_cover: <$Sub_name>\n";
     return unless defined $Sub_name;  # Only happens within Safe.pm, AFAIK.
 
     get_location($start) if $start;
-    # print "[[$File:$Line]]\n";
+    # print STDERR "[[$File:$Line]]\n";
     # return unless length $File;
     return if length $File && !use_file($File);
 
@@ -1088,7 +1102,7 @@ sub get_cover
                 }
             }
             $Pod = "Pod::Coverage" if delete $opts{nocp};
-            # use Data::Dumper; print "$Pod, ", Dumper \%opts;
+            # use Data::Dumper; print STDERR "$Pod, ", Dumper \%opts;
             if ($Pod{$file} ||= $Pod->new(package => $pkg, %opts))
             {
                 my $covered;
@@ -1119,9 +1133,9 @@ sub get_cover
     # my $dd = @_ && ref $_[0]
                  # ? $deparse->deparse($_[0], 0)
                  # : $deparse->deparse_sub($cv, 0);
-    # print "get_cover: <$Sub_name>\n";
-    # print "[[$File:$Line]]\n";
-    # print "<$dd>\n";
+    # print STDERR "get_cover: <$Sub_name>\n";
+    # print STDERR "[[$File:$Line]]\n";
+    # print STDERR "<$dd>\n";
 
     no warnings "redefine";
     local *B::Deparse::deparse     = \&deparse;
@@ -1131,7 +1145,7 @@ sub get_cover
     my $de = @_ && ref $_[0]
                  ? $deparse->deparse($_[0], 0)
                  : $deparse->deparse_sub($cv, 0);
-    # print "<$de>\n";
+    # print STDERR "<$de>\n";
     $de
 }
 
@@ -1393,11 +1407,11 @@ See the BUGS file.  And the TODO file.
 
 =head1 VERSION
 
-Version 0.58 - 6th August 2006
+Version 0.60 - 2nd January 2007
 
 =head1 LICENCE
 
-Copyright 2001-2006, Paul Johnson (pjcj@cpan.org)
+Copyright 2001-2007, Paul Johnson (pjcj@cpan.org)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
